@@ -1,15 +1,43 @@
+-- | This is the main module of console-program. You can use it to build a
+-- console program, that parses the command line (and a configuration file),
+-- divides it into modes/commands, options and non-options, and executes the
+-- corresponding action from a tree of commands.
+-- 
+-- The main function is 'execute'; provided with a tree of commands and a
+-- default configuration, it can be used as the @main@ function.
+-- 
+-- A \"mode\" or \"command\" provides a mode of operation of your program.
+-- This allows a single executable to provide many different pieces of
+-- functionality. The first argument to the program (or the first few, if it
+-- has subcommands) determines which command should be executed.
+-- (@darcs@ and @cabal@ are examples of programs that make heavy use of modes.)
+-- 
+-- Options can be given in a configuration file, and on the command line.
+-- Commands specify which options apply to them (the 'applicableOptions' field).
+-- Such option descriptions can be created by 'System.Console.Argument.option'.
+-- 
+-- Options can have arguments, as in @program --option=value@, where @value@ is
+-- the argument to @option@. These arguments have types, dictated by the
+-- particular option. These types are represented by a 'System.Console.Argument.Type'.
+-- 
+-- A command can have also non-option arguments (plain arguments). These also
+-- have types. Create such commands using the function 'System.Console.Action.withArgument'.
 module System.Console.Command
   (
     Commands
   , Command (Command,name,applicableNonOptions,applicableOptions,description,action)
-  
-  , single
+
+  -- * Using a command tree to construct a program
+  , execute
   , showUsage
+  
+  -- * Configuration file
+  -- $configfile
   ) where
 
 
-import qualified System.Console.Action  as Action
-import qualified System.Console.Options as Options
+import qualified System.Console.Action.Internal as Action
+import qualified System.Console.Options         as Options
 
 import           Control.Applicative     ((<$>))
 import           Control.Arrow           ((&&&),second)
@@ -34,19 +62,38 @@ import qualified Text.PrettyPrint.ANSI.Leijen as PP
 type Commands setting
   = Tree.Tree (Command setting)
 
--- | A @Command s@ is an action, together with some descriptive information:
--- name (this is the textual command that invokes the action), description,
--- and lists of applicable options and non-options. @s@ is the type of setting.
-data Command s
+-- | A @Command s@ is an action, together with some descriptive information.
+-- The description, and lists of applicable options and non-options are
+-- used only to show usage information for the command.
+-- @s@ is the type of settings that the action may use.
+data Command c
   = Command
     {
       name :: String
+      -- ^ This determines which command is executed, depending on the command line.
     , applicableNonOptions :: [String]
-    , applicableOptions :: [GetOpt.OptDescr (Either String s)]
+      -- ^ For usage info.
+    , applicableOptions :: [GetOpt.OptDescr (Either String (Options.Setting c))]
+      -- ^ For usage info; also, the union of this field, over all commands in
+      -- the command tree, determines which options will be recognised when
+      -- parsing the configuration file and command line.
     , description :: String
-    , action :: Action.Action (Options.Options s)
+      -- ^ For usage info.
+    , action :: Action.Action c
     }
 
+-- $configfile
+-- The configuration file is assumed to be in the user's home directory, and
+-- to be named \".foobar\", where \"foobar\" is the name of the command at the
+-- root of the command tree (usually the name of the program).
+-- 
+-- Settings in this file are of the form
+-- @
+--   option-name=option-value
+-- @
+-- , see module "Fez.Data.Conf" for details. The format of the \"option-value\"
+-- part depends on the type of the option argument; see "System.Console.Argument".
+
 fromDisk :: String -> IO [String]
 fromDisk configFile = do
   home <- getHomeDirectory
@@ -55,7 +102,7 @@ fromDisk configFile = do
 
 -- Load configuration file and parse the command line into settings
 -- and non-options.
-load :: Commands s -> IO ([String],[s])
+load :: Commands c -> IO ([String],[Options.Setting c])
 load commands = do
   let options = Set.toList $ collectOptions commands
   fileArgs <- fromDisk $ '.' : name (Tree.rootLabel commands)
@@ -65,18 +112,25 @@ load commands = do
     else traverse putStrLn errors >> showUsage commands >> exitFailure
 
 -- | Load the configuration file (if present), and run the action given on the command line.
-single :: (Options.Setting s) => Options.Options s -> Commands s -> IO ()
-single defaults commands = uncurry (select commands) . second (flip Options.apply defaults) =<< load commands
+-- You may use this function, with appropriate arguments, as your @main@ function.
+-- 
+-- Settings in the configuration file override the default configuration;
+-- settings on the command line override both.
+execute :: (Options.Configuration c) =>
+     c          -- ^ Default configuration.
+  -> Commands c -- ^ Tree of commands.
+  -> IO ()
+execute defaults commands = uncurry (select commands) . second (flip Options.apply defaults) =<< load commands
 
 -- Select the right command from the command tree, given a list of non-options, and the options.
-select :: (Options.Setting s) => Commands s -> [String] -> Options.Options s -> IO ()
+select :: (Options.Configuration c) => Commands c -> [String] -> c -> IO ()
 select (Tree.Node root _     ) []           = Action.run (action root) []
 select (Tree.Node root forest) nos@(x : xs) = case lookup x $ map (name . Tree.rootLabel &&& id) forest of
   Nothing -> Action.run (action root) nos
   Just cs -> select cs xs
 
--- | Show usage info for the program.
-showUsage :: Commands o -> IO ()
+-- | Print usage info for the program to stdout.
+showUsage :: Commands c -> IO ()
 showUsage = PP.putDoc . usage
  where
   usage (Tree.Node c ns) = subcs ns . (PP.<> PP.line) . opts c . descr c . nonOpts c $ PP.bold (PP.text $ name c)
@@ -96,7 +150,7 @@ showUsage = PP.putDoc . usage
   list i p a = PP.fill i . PP.cat . PP.punctuate PP.comma . map (\ x -> PP.text p PP.<> PP.text x PP.<> a)
   subcs ns = if null ns then id else flip (PP.<$>) $ PP.indent 2 (PP.vsep $ map usage ns)
 
-collectOptions :: Commands o -> Set.Set (GetOpt.OptDescr (Either String o))
+collectOptions :: Commands c -> Set.Set (GetOpt.OptDescr (Either String (Options.Setting c)))
 collectOptions = foldMap (Set.fromList . applicableOptions)
 
 instance Eq (GetOpt.OptDescr a) where
